@@ -12,6 +12,8 @@ import matplotlib.pyplot as plt
 import seaborn as sns
 import numpy as np
 import matplotlib_venn as mpv
+import math
+import scipy.stats
 
 ALPHA = 0.8
 MARKER_SIZE = 10
@@ -45,39 +47,81 @@ def setup():
   sns.set_style({"axes.facecolor": 'none'})
 
 
-def new_ax(fig, *args, **kwargs):
-    zorder = kwargs.get('zorder', 1)
-    sharex = kwargs.get('sharex', None)
-    sharey = kwargs.get('sharey', None)
+
+def format_axes(ax, x='', y='', direction='in'):
+    ax.set_xlabel(x)
+    ax.set_ylabel(y)
+    ax.spines['right'].set_visible(False)
+    ax.spines['top'].set_visible(False)
+    ax.minorticks_on()
+    ax.get_yaxis().set_tick_params(which='both', direction=direction)
+    ax.get_xaxis().set_tick_params(which='both', direction=direction)
     
-    if len(args) == 3:
-        ax = fig.add_subplot(args[0], args[1], args[2], zorder=zorder, sharex=sharex, sharey=sharey)
-    else:
-        subplot = kwargs.get('subplot', '111')
-        
-        if type(subplot) is tuple:
-            ax = fig.add_subplot(subplot[0], subplot[1], subplot[2], zorder=zorder, sharex=sharex, sharey=sharey)
+def invisible_axes(ax):
+    """
+    Make axes invisible.
+    
+    Parameters
+    ----------
+    ax :
+        Matplotlib ax object.
+    """
+    
+    ax.xaxis.set_visible(False)
+    ax.yaxis.set_visible(False)
+    ax.spines['right'].set_visible(False)
+    ax.spines['top'].set_visible(False)
+    ax.spines['bottom'].set_visible(False)
+    ax.spines['left'].set_visible(False)
+
+def new_ax(fig, subplot=(1,1,1), zorder=1, root_ax=None, sharex=None, sharey=None, direction='in'):
+    if isinstance(subplot, str):
+        if ':' in subplot:
+            subplot = tuple(int(c) for c in subplot.split(':'))
         else:
-            ax = fig.add_subplot(subplot, zorder=zorder, sharex=sharex, sharey=sharey)
-  
-    format_axes(ax)
+            subplot = tuple(int(c) for c in subplot)
+    elif isinstance(subplot, int):
+        subplot = tuple((subplot // (10 ** i)) % 10 for i in range(math.ceil(math.log(subplot, 10)) - 1, -1, -1))
     
+    if root_ax is not None:
+        sharex=root_ax
+        sharey=root_ax
+        zorder=root_ax.zorder + 10
+    
+    ax = fig.add_subplot(subplot[0], subplot[1], subplot[2], zorder=zorder, sharex=sharex, sharey=sharey)
+  
+    ax.patch.set_alpha(0)
+    
+    if root_ax is not None:
+        
+        invisible_axes(ax)
+    else:
+        format_axes(ax, direction=direction)
+        
     return ax
 
+
 def new_base_fig(w=8, h=8):
-    fig = plt.figure(figsize=[w, h])
+    if isinstance(w, tuple):
+        h = w[1]
+        w = w[0]
+        
+    fig = plt.figure(figsize=(w, h))
     
     return fig
 
-def new_fig(w=8, h=8, subplot=111):
+
+def newfig(w=8, h=8, subplot=(1,1,1), direction='in'):
+    return new_fig(w=w, h=h, subplot=subplot, direction=direction)
+
+def new_fig(w=8, h=8, subplot=111, direction='in'):
     fig = new_base_fig(w, h)
     
-    ax = new_ax(fig, subplot)
+    ax = new_ax(fig, subplot, direction=direction)
   
-    format_axes(ax)
-    
     return fig, ax
 
+
 def grid_size(n):
     return int(np.ceil(np.sqrt(n)))
 
@@ -106,6 +150,10 @@ def polar_clock_ax(fig, subplot=111):
     return ax
 
 
+def format_legend(ax, cols=6, markerscale=None):
+  ax.legend(bbox_to_anchor=[0, 0.95], loc='lower left', ncol=cols, frameon=False, fontsize='small', markerscale=markerscale, handlelength=1, columnspacing=0.5)
+
+
 
 def savefig(fig, out, pad=2, dpi=300):
   fig.tight_layout(pad=pad) #rect=[o, o, w, w])
@@ -158,8 +206,6 @@ def base_boxplot(df, x=None, y=None, hue=None, width=0.1, colors=BLUES, linewidt
     
     colors = get_tint(colors, tint)
     
-    print(colors)
-    
     sns.boxplot(x=x, y=y, hue=hue, data=df, width=width, palette=colors, fliersize=fliersize, linewidth=linewidth, orient="v", saturation=1, ax=ax)
       
     for i in range(0, len(ax.lines)):
@@ -192,11 +238,13 @@ def boxplot(df, x=None, y=None, hue=None, width=0.1, colors=BLUES, linewidth=1.5
     ax = base_boxplot(x=x, y=y, hue=hue, df=df, width=width, colors=colors, linewidth=linewidth, fliersize=fliersize, orient=orient, tint=tint, ax=ax)
       
     format_axes(ax, x=x, y=y)
+    
+    ax.tick_params(axis='x',which='minor',bottom='off')
       
     return ax
 
 
-def base_violinplot(df, x=None, y=None, hue=None, width=0.4, colors=BLUES[0], tint=0, ax=None):
+def base_violinplot(df, x=None, y=None, hue=None, width=0.4, colors=BLUES, tint=0, fig=None, ax=None):
     if ax is None:
         fig, ax = new_fig()
         
@@ -207,16 +255,44 @@ def base_violinplot(df, x=None, y=None, hue=None, width=0.4, colors=BLUES[0], ti
     sns.violinplot(x=x, y=y, hue=hue, data=df, width=width, palette=colors, linewidth=0, orient='v', saturation=1, ax=ax)
       
     format_axes(ax, x=x, y=y)
+    
+    ax.tick_params(axis='x',which='minor',bottom='off')
       
-    return ax
+    return fig, ax
 
 
-def violinplot(df, x=None, y=None, hue=None, width=0.4, colors=BLUES[0], tint=0, ax=None):
-    ax = base_violinplot(df, x=x, y=y, hue=hue, width=width, colors=colors, tint=tint, ax=ax)
+def violinplot(df, x=None, y=None, hue=None, width=0.4, colors=BLUES, tint=0, ax=None):
+    fig, ax = base_violinplot(df, x=x, y=y, hue=hue, width=width, colors=colors, tint=tint, ax=ax)
       
     format_axes(ax, x=x, y=y)
       
-    return ax
+    return fig, ax
+
+def violinboxplot(df, x=None, y=None, hue=None, width=0.6, colors=BLUES, fig=None, wf=1.5):
+    if fig is None:
+        if x is not None:
+            w = np.unique(df[x]).size
+            print(w)
+        else:
+            w = df.shape[1]
+        
+        w *= wf
+        
+        fig = new_base_fig(w=w, h=6)
+        
+    ax = new_ax(fig)
+    
+    base_violinplot(df, x=x, y=y, hue=hue, width=width, colors=colors, tint=0.5, ax=ax)
+    
+    format_axes(ax, x=x, y=y)
+    
+    ax2 = new_ax(fig, root_ax=ax)
+    
+    base_boxplot(df, x=x, y=y, hue=hue, width=width/4, colors=colors, ax=ax2)
+      
+    #invisible_axes(ax2)
+      
+    return fig
 
 
 def scatter(x, y, s=MARKER_SIZE, c=None, cmap=None, norm=None, alpha=ALPHA, marker='o', fig=None, ax=None, label=None):
@@ -248,11 +324,11 @@ def correlation_plot(x, y, marker='o', s=MARKER_SIZE, c=None, cmap=None, norm=No
     return fig, ax
 
 
-def plot(x, y, s=MARKER_SIZE, c=None, alpha=ALPHA, fig=None, ax=None, label=''):
+def plot(x, y, s=MARKER_SIZE, c=None, alpha=ALPHA, fig=None, ax=None, linewidth=1, label='', solid_capstyle='round'):
     if ax is None:
         fig, ax = new_fig()
         
-    gcf = ax.plot(x, y, c=c, alpha=alpha, label=label)
+    gcf = ax.plot(x, y, c=c, alpha=alpha, label=label, linewidth=linewidth, solid_capstyle=solid_capstyle)
     
     return fig, ax, gcf
 
@@ -285,36 +361,13 @@ def venn2(s1, s2, l1, l2, fig=None, ax=None):
     
     return fig, ax, v
 
-def invisible_axes(ax):
-    """
-    Make axes invisible.
-    
-    Parameters
-    ----------
-    ax :
-        Matplotlib ax object.
-    """
-    
-    ax.xaxis.set_visible(False)
-    ax.yaxis.set_visible(False)
-    ax.spines['right'].set_visible(False)
-    ax.spines['top'].set_visible(False)
-    ax.spines['bottom'].set_visible(False)
-    ax.spines['left'].set_visible(False)
+
   
 
-def format_axes(ax, x='', y=''):
-    ax.set_xlabel(x)
-    ax.set_ylabel(y)
-    ax.spines['right'].set_visible(False)
-    ax.spines['top'].set_visible(False)
-    ax.minorticks_on()
-    ax.get_yaxis().set_tick_params(which='both', direction='in')
-    ax.get_xaxis().set_tick_params(which='both', direction='in')
 
 
 def add_colorbar(fig, cmap, x1=None, x2=None, norm=None):
-    cax = fig.add_axes([0.8, 0.1, 0.15, 0.02])
+    cax = fig.add_axes([0.8, 0.1, 0.15, 0.02], zorder=100)
     cb = matplotlib.colorbar.ColorbarBase(cax, cmap=cmap, ticks=[0, 1.0], orientation='horizontal')
     
     if x1 is None:
@@ -371,4 +424,79 @@ def get_tint(colors, t):
             
         return ret
     else:
-        return colors
\ No newline at end of file
+        return colors
+    
+    
+    
+def bin_data(x, y, bins=100):
+    ys = np.empty(0)
+    xs = np.empty(0)
+    xs_std = np.empty(0)
+    
+    idx = 0
+    
+    tcurrent = 0
+    tend = x.max()
+    tinc = (tend - x.min()) / bins
+    
+    n = x.shape[0]
+    
+    while tcurrent < tend:
+        #print(idx)
+        tnext = tcurrent + tinc
+
+        # find all points in the bin
+        e = y[(x >= tcurrent) & (x < tnext)]
+        
+        #print(e)
+        
+        if e.size > 0:
+            #m,_ = scipy.stats.mode(e)
+            #m = m[0]
+            
+            #print(e)
+            #print(tcurrent, e.mean())
+            m = np.mean(e)
+            ys = np.append(ys, m)
+            xs_std = np.append(xs_std, np.std(e))
+        else:
+            ys = np.append(ys, 0)
+            xs_std = np.append(xs_std, 0)
+        
+        xs = np.append(xs, tcurrent)
+        
+        tcurrent = tnext
+        
+    return xs, ys
+
+
+def color_labels(ax, colors, indices, color_indices=None):
+    """
+    Color labels on an axes object.
+    
+    Parameters
+    ----------
+    ax : matplotlib axes
+        Axes to adjust
+    colors : list or array
+        List of colors to use for labelling
+    indices : list or int
+        List of label indices to color. If indices is an int, it will be
+        interpreted as a range between 0 and n (exclusive).
+    """
+    
+    if isinstance(indices, int):
+        indices = np.array([i for i in range(0, indices)])
+        
+    if not isinstance(indices, np.ndarray):
+        indices = np.array(indices)
+        
+    if color_indices is None:
+        color_indices = indices
+        
+    if not isinstance(color_indices, np.ndarray):
+        color_indices = np.array(color_indices)
+    
+    for i in range(0, indices.size):
+        t = ax.get_yticklabels()[indices[i]]
+        t.set_color(colors[color_indices[i]])
\ No newline at end of file
